'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (babel) {
  var t = babel.types,
      template = babel.template;

  var identifiers = new Set();
  var buildBuiltInWithConfig = template(`
    GLAMOROUS.BUILT_IN.withConfig({displayName: DISPLAY_NAME})
  `);
  var buildCustomWithConfig = template(`
    GLAMOROUS(ARGUMENTS).withConfig({displayName: DISPLAY_NAME})
  `);
  return {
    name: 'babel-plugin-glamorous-displayName',
    visitor: {
      ImportDeclaration(path) {
        var defaultSpecifierPath = path.get('specifiers')[0];
        if (path.node.source.value !== 'glamorous' || !t.isImportDefaultSpecifier(defaultSpecifierPath)) {
          return;
        }
        var name = defaultSpecifierPath.node.local.name;

        var _path$scope$getBindin = path.scope.getBinding(name),
            referencePaths = _path$scope$getBindin.referencePaths;

        referencePaths.forEach(function (reference) {
          identifiers.add(reference);
        });
      },
      VariableDeclarator(path) {
        var node = path.node;

        if (!isRequireCall(node.init) || !t.isIdentifier(node.id)) {
          return;
        }
        var name = node.id.name;

        var binding = path.scope.getBinding(name);
        // I'm not sure whether this could ever happen
        // but I don't want a bug report hitting me because it did...
        // istanbul ignore else
        if (binding) {
          var referencePaths = binding.referencePaths;

          referencePaths.forEach(function (reference) {
            identifiers.add(reference);
          });
        }
      },
      Program: {
        exit(path, _ref) {
          var file = _ref.file;

          Array.from(identifiers).forEach(function (identifier) {
            var displayName = getDisplayName(identifier, file);

            handleBuiltIns(identifier, displayName);
            handleCustomComponent(identifier, displayName);
          });
        }
      }
    }
  };

  function handleBuiltIns(path, displayName) {
    var isBuiltIn = looksLike(path, {
      parentPath: {
        type: 'MemberExpression',
        node: {
          property: {
            type: 'Identifier'
          }
        },
        parent: {
          type: 'CallExpression'
        }
      }
    });
    if (!isBuiltIn) {
      return;
    }
    path.parentPath.replaceWith(buildBuiltInWithConfig({
      GLAMOROUS: path.node,
      BUILT_IN: path.parent.property,
      DISPLAY_NAME: t.stringLiteral(displayName)
    }));
  }

  function handleCustomComponent(path, displayName) {
    var isCustom = looksLike(path, {
      parent: {
        type: 'CallExpression'
      }
    });
    if (!isCustom) {
      return;
    }
    path.parentPath.replaceWith(buildCustomWithConfig({
      GLAMOROUS: path.node,
      ARGUMENTS: path.parent.arguments,
      DISPLAY_NAME: t.stringLiteral(displayName)
    }));
  }

  // credit: https://github.com/styled-components/babel-plugin-styled-components/blob/37a13e9c21c52148ce6e403100df54c0b1561a88/src/visitors/displayNameAndId.js
  function getDisplayName(path, file) {
    var componentName = getName(path);
    var filename = getFileName(file);
    if (filename) {
      if (filename === componentName) {
        return componentName;
      }
      return componentName ? `${filename}__${componentName}` : filename;
    } else {
      return componentName;
    }
  }

  // credit: https://github.com/styled-components/babel-plugin-styled-components/blob/37a13e9c21c52148ce6e403100df54c0b1561a88/src/utils/getName.js
  function getName(path) {
    var namedNode = void 0;

    path.find(function (parentPath) {
      if (parentPath.isObjectProperty()) {
        // const X = { Y: glamorous }
        namedNode = parentPath.node.key;
      } else if (parentPath.isVariableDeclarator()) {
        // let X; X = glamorous
        namedNode = parentPath.node.id;
      } else if (parentPath.isStatement()) {
        // we've hit a statement, we should stop crawling up
        return true;
      }

      // we've got an displayName (if we need it) no need to continue
      if (namedNode) {
        return true;
      }
      return false;
    });

    // identifiers are the only thing we can reliably get a name from
    return t.isIdentifier(namedNode) ? namedNode.name : undefined;
  }
};

var nodePath = require('path');
// const nodePath = {} // handy when you wanna copy this into astexplorer.net

function getFileName(file) {
  if (!file || file.opts.filename === 'unknown') {
    return '';
  }
  return file.opts.basename === 'index' ? nodePath.basename(nodePath.dirname(file.opts.filename)) : file.opts.basename;
}

function isRequireCall(callExpression) {
  return looksLike(callExpression, {
    type: 'CallExpression',
    callee: {
      name: 'require'
    },
    arguments: function _arguments(args) {
      return args && args.length === 1 && args[0].value === 'glamorous';
    }
  });
}

function looksLike(a, b) {
  return a && b && Object.keys(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === 'function') {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  // eslint-disable-next-line
  return val == null || /^[sbn]/.test(typeof val);
}